package com.fzm.mall.server.front.order.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fzm.mall.server.front.asset.model.OrderPay;
import com.fzm.mall.server.front.asset.model.UserAsset;
import com.fzm.mall.server.front.asset.model.UserAssetRecord;
import com.fzm.mall.server.front.asset.model.po.qo.OrderQO;
import com.fzm.mall.server.front.asset.model.vo.order.OrderDetailVO;
import com.fzm.mall.server.front.asset.model.vo.order.OrderVO;
import com.fzm.mall.server.front.asset.service.IOrderPayService;
import com.fzm.mall.server.front.asset.service.IUserAssetRecordService;
import com.fzm.mall.server.front.asset.service.IUserAssetService;
import com.fzm.mall.server.front.base.PageVO;
import com.fzm.mall.server.front.constant.MALLGlobalConfig;
import com.fzm.mall.server.front.constant.PaymentTypeConst;
import com.fzm.mall.server.front.constant.cache.MallCacheHeader;
import com.fzm.mall.server.front.enums.QueueTTLTypeEnum;
import com.fzm.mall.server.front.goods.model.*;
import com.fzm.mall.server.front.goods.model.vo.GoodsVo;
import com.fzm.mall.server.front.goods.model.vo.SkuVo;
import com.fzm.mall.server.front.goods.service.*;
import com.fzm.mall.server.front.merchant.model.Merchant;
import com.fzm.mall.server.front.merchant.service.IMerchantService;
import com.fzm.mall.server.front.mq.MsgProducer;
import com.fzm.mall.server.front.order.mapper.OrderSkuMapper;
import com.fzm.mall.server.front.order.mapper.RefundMapper;
import com.fzm.mall.server.front.order.model.*;
import com.fzm.mall.server.front.order.mapper.OrderMapper;
import com.fzm.mall.server.front.order.model.po.NftOrderPo;
import com.fzm.mall.server.front.order.model.po.PreSaleOrderVo;
import com.fzm.mall.server.front.order.model.po.TranDto;
import com.fzm.mall.server.front.order.model.vo.*;
import com.fzm.mall.server.front.order.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.mall.server.front.redis.OrderRedis;
import com.fzm.mall.server.front.repository.RedisRepository;
import com.fzm.mall.server.front.user.model.User;
import com.fzm.mall.server.front.user.service.IUserService;
import com.fzm.mall.server.front.util.ChainUtil;
import com.fzm.mall.server.front.util.OrderUtil;
import com.fzm.mall.server.front.util.TimeUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 用户购物订单表 服务实现类
 * </p>
 *
 * @author fzm
 * @since 2021-03-02
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {
    private final OrderRedis orderRedis;
    private final IMerchantService merchantService;
    private final IGoodSkuService goodSkuService;
    private final IGoodSpuService spuService;
    private final IGoodSkuPropService skuPropService;
    private final IOrderDetailService orderDetailService;
    private final OrderMapper orderMapper;
    private final RedisRepository redisRepository;
    private final OrderSkuMapper orderSkuMapper;
    private final IMarketingCouponService marketingCouponService;
    private final IMarketingPromotionService marketingPromotionService;
    private final IDistributionTemplateService distributionTemplateService;
    private final IPayService payService;
    private final IUserAssetService userAssetService;
    private final ICountService countService;
    private final IDeliveryService deliveryService;
    private final IGoodsCouponService goodsCouponService;
    private final IOrderPayService orderPayService;
    private final IGoodCommentService goodCommentService;
    private final RefundMapper refundMapper;
    private final ISkuService skuService;
    private final IUserAssetRecordService userAssetRecordService;
    private final IGoodsPromotionService promotionService;
    private final IGoodsCouponService couponService;
    private final IPreSaleService preSaleService;
    private final MsgProducer msgProducer;
    private final IUserService userService;
    private final INftService nftService;
    private final ISpuService ispuService;
    @Autowired
    private ChainUtil chainUtil;


    /**
     * 查询订单
     */
    @Override
    public Order order(final String oid) {
        Order order = orderRedis.getOrder(oid);
        if (order != null) {
            return order;
        }
        QueryWrapper<Order> qw = new QueryWrapper<>();
        qw.eq("oid", oid);
        order = getOne(qw);
        if (order != null) {
            orderRedis.setOrder(oid, order);
        }
        return order;
    }

    /**
     * 查询出订单以及商家名称
     */
    @Override
    public OrderVO orderVO(final String oid) {
        Order order = order(oid);
        OrderVO orderVO = new OrderVO();
        if (order != null) {
            BeanUtils.copyProperties(order, orderVO);
            String merchantId = order.getMerchantId();
            OrderPay orderPay = orderPayService.getByPayId(orderVO.getPayId());
            orderVO.setPayType(orderPay.getPayType());
            orderVO.setPayTime(orderPay.getPayTime());
            Merchant info = merchantService.info(merchantId);
            if (info != null) {
                orderVO.setEnterpriseName(info.getEnterpriseName());
            }
            orderVO.setMerchantId(merchantId);
            GoodsPromotion promotion = promotionService.getByOid(oid);
            if (promotion != null) {
                orderVO.setPromotionAmount(promotion.getAmount().setScale(2, BigDecimal.ROUND_DOWN));
            }
            GoodsCoupon goodsCoupon = couponService.getByOid(oid);
            if (goodsCoupon != null) {
                orderVO.setCouponAmount(goodsCoupon.getAmount().setScale(2, BigDecimal.ROUND_DOWN));
            }
            buildOrderVO(orderVO);
        }
        return orderVO;
    }

    /**
     * 构建OrderVO
     */
    @Override
    public void buildOrderVO(OrderVO orderVO) {
        String oid = orderVO.getOid();
        List<OrderDetail> listOrderDetail = orderDetailService.listOrderDetail(oid);
        List<OrderDetailVO> orders = new ArrayList<>(listOrderDetail.size());
        for (OrderDetail po : listOrderDetail) {
            OrderDetailVO vo = new OrderDetailVO();
            BeanUtils.copyProperties(po, vo);
            String gid = po.getGoodsId();
            String skuId = po.getSkuId();
            GoodSpu goodSpu = spuService.goodSpu(gid);
            Integer type = goodSpu.getType();
            vo.setType(type);
            if (type == 2) {
                //预售商品
                PreSale preSale = preSaleService.getByGoodsId(gid);
                vo.setPreSale(preSale);
            }
            vo.setThumb(goodSpu.getThumb());
            String commonentId = goodCommentService.getCommonentId(oid, skuId);
            vo.setCommentId(commonentId);
            List<Map<String, Object>> skuPropListMap = skuPropService.getSkuPropListBySkuId(skuId);
            vo.setPropList(skuPropListMap);
            orders.add(vo);
        }
        orderVO.setOrders(orders);
    }

    /**
     * 分页查询订单
     */
    @Override
    public PageVO<OrderVO> pageOrderVO(final OrderQO orderQO) {
        List<OrderVO> listOrder = orderMapper.listOrder(orderQO);
        Integer totalSize = orderMapper.countOrder(orderQO);
        if (!CollectionUtils.isEmpty(listOrder)) {
            for (OrderVO orderVO : listOrder) {
                buildOrderVO(orderVO);
            }
        }
        PageVO<OrderVO> pageVO = new PageVO<>();
        pageVO.init(orderQO.getPage(), orderQO.getSize(), totalSize, listOrder);
        return pageVO;
    }

    /**
     * 分页查询售后订单
     */
    @Override
    public PageVO<OrderVO> pageAfterSaleOrderVO(final OrderQO orderQO) {
        List<OrderVO> listOrder = orderMapper.listAfterSaleOrder(orderQO);
        Integer totalSize = orderMapper.countAfterSaleOrder(orderQO);
        if (!CollectionUtils.isEmpty(listOrder)) {
            for (OrderVO orderVO : listOrder) {
                buildOrderVO(orderVO);
            }
        }
        PageVO<OrderVO> pageVO = new PageVO<>();
        pageVO.init(orderQO.getPage(), orderQO.getSize(), totalSize, listOrder);
        return pageVO;
    }

    /**
     * 处理订单
     * TODO 订单处理可能发生错误，可以做一些收集
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void process(String oid) {
        // 查询订单状态
        Order orderPO = order(oid);
        if (orderPO == null || PaymentTypeConst.PAY_WAIT != orderPO.getOrderState()) {
            log.debug("订单不存在或订单不是待支付状态：{}" + oid);
            return;
        }
        // 订单超时
        long timestamp = TimeUtil.timestamp();
        Integer ok = orderMapper.orderTimeout(oid, timestamp);
        if (ok == null || ok != 1) {
            log.warn("订单超时失败：{}" + oid);
            return;
        }
        log.debug("订单超时成功：{}" + oid);
        List<OrderDetail> listOrderDetail = listOrderDetail(oid);
        for (OrderDetail po : listOrderDetail) {
            String gid = po.getGoodsId();
            // 查询商品计数方式
            //GoodSale goodSale = goodSaleService.goodSale(gid);
            // 拍下减库存  目前都是拍下立减
            // 恢复商品数量
            ok = orderSkuMapper.goodRestore(gid, po.getCoin(), po.getNumber().toString());
            if (ok == null || ok != 1) {
                log.warn("恢复商品数量失败：{}" + oid);
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return;
            }
            log.debug("恢复商品数量成功：{}" + oid);
            redisRepository.delObj(MallCacheHeader.goods_sku + gid);
        }
        // 删除缓存
        redisRepository.delObj(MallCacheHeader.GOOD_ORDER + oid);
    }

    /**
     * 订单详情列表
     */
    @Override
    public List<OrderDetail> listOrderDetail(final String oid) {
        try {
            Object obj = redisRepository.getObj(MallCacheHeader.GOOD_ORDER_DETAIL + oid);
            if (obj != null) {
                return JSON.parseArray(JSON.toJSONString(obj), OrderDetail.class);
            }
        } catch (Exception e) {
            // 缓存中获取的数据转换发生错误，重置缓存
            redisRepository.delObj(MallCacheHeader.GOOD_ORDER_DETAIL + oid);
        }
        List<OrderDetail> listOrderDetail = orderDetailService.listOrderDetail(oid);
        if (!CollectionUtils.isEmpty(listOrderDetail)) {
            redisRepository.setObj(MallCacheHeader.GOOD_ORDER_DETAIL + oid, listOrderDetail, 7, TimeUnit.HOURS);
        }
        return listOrderDetail;
    }

    //创建订单
    @Transactional
    @Override
    public String createOrder(String uid, OrderVo orderVo, Integer type) {
        //总价格
        BigDecimal all = BigDecimal.ZERO;
        List<MerchantGoodsVo> merchantGoodsVos = orderVo.getMerchantGoodsVos();
        //支付单
        Pay pay = new Pay();
        pay.setPayId(OrderUtil.getPayId());
        JSONArray a = new JSONArray();
        Map<String, Order> merchantOrder = new HashMap<>();
        //所有skuId
        List<OrderSkuVo> allSkuVo = new ArrayList<>();
        for (MerchantGoodsVo m : merchantGoodsVos) {
            //订单
            Order order = new Order();
            String oid = OrderUtil.getOid();
            order.setOid(oid);
            List<OrderSkuVo> orderSkuVos = m.getOrderSkuVos();
            //订单总价
            BigDecimal total = BigDecimal.ZERO;
            //快递模板id
            String templateId = "";
            //每家店对应商品数量
            Integer merchantGoodSize = 0;
            //每家店对应商品重量
            BigDecimal merchantGoodWeight = BigDecimal.ZERO;


            for (OrderSkuVo orderSkuVo : orderSkuVos) {
                SkuVo skuVo = goodSkuService.getSkuVoBySkuId(orderSkuVo.getSkuId());
                if (skuVo == null || skuVo.getStatus() != 1) {
                    //商品不存在或下架
                    throw new RuntimeException("商品不存在或下架");
                }
                if (orderSkuVo.getNum() < 1) {
                    //商品数量错误
                    throw new RuntimeException("商品数量错误");
                }
                int i = goodSkuService.delStock(skuVo.getSkuId(), orderSkuVo.getNum());
                if (i != 1) {
                    throw new RuntimeException("库存不足!!!");
                }
                //统计每家店对应商品数量
                merchantGoodSize = merchantGoodSize + orderSkuVo.getNum();
                //统计每家店对应商品重量
                //merchantGoodWeight = merchantGoodWeight.add(skuVo.getWeight().multiply(new BigDecimal(orderSkuVo.getNum())));
                templateId = skuVo.getTemplateId();
                OrderDetail orderDetail = new OrderDetail();
                orderDetail.setOid(order.getOid());
                orderDetail.setGoodsId(skuVo.getGoodsId());
                orderDetail.setSkuId(skuVo.getSkuId());
                orderDetail.setCoin(skuVo.getCoinName());
                orderDetail.setName(skuVo.getName());
                orderDetail.setNumber(orderSkuVo.getNum());
                orderDetail.setAmount(skuVo.getOriginalPrice());
                orderDetailService.save(orderDetail);
                all = all.add(skuVo.getOriginalPrice().multiply(new BigDecimal(orderSkuVo.getNum())));
                total = total.add(skuVo.getOriginalPrice().multiply(new BigDecimal(orderSkuVo.getNum())));
            }
            /*//邮费计算
            BigDecimal PostFee = BigDecimal.ZERO;
            if (orderVo.getDelivery() == 0) {

            } else {
                PostFee = distributionTemplateService.getPrice(templateId, orderVo.getProvince(), total, merchantGoodSize, merchantGoodWeight);
            }

            if (PostFee.compareTo(new BigDecimal("-1")) == 0) {
                throw new RuntimeException("当前地区不支持快递");
            }
            all = all.add(PostFee);
            allSkuVo.addAll(orderSkuVos);
            //计算促销
            BigDecimal price = marketingPromotionService.promotion(m.getMerchantId(), orderSkuVos, type, oid);
            all = all.subtract(price);*/
            order.setUid(uid);
            order.setPayId(pay.getPayId());
            order.setMerchantId(m.getMerchantId());
            order.setTotal(total);
            order.setOrderType(1);
            //order.setDelivery(orderVo.getDelivery());
            order.setDelivery(0);
            order.setUname(orderVo.getUname());
            order.setPhone(orderVo.getPhone());
            order.setAddress(orderVo.getAddress());
            order.setVNote(m.getVNote());
            //order.setAmount(merchantAll.subtract(mermantPromotion).subtract(mermantCoupon).add(PostFee));
            //order.setDiscountAmount(promotionAll.add(couponAll));
            order.setPostFee(BigDecimal.ZERO);
            order.setDiscountAmount(BigDecimal.ZERO);
            order.setAmount(total);
            order.setOrderState(7);
            a.add(oid);
            orderMapper.insert(order);
            merchantOrder.put(m.getMerchantId(), order);
        }
        //
       /* List<String> goodsIdCouponVos = orderVo.getGoodsIdCouponVos();
        Map<String, BigDecimal> merchantCoupon = new HashMap<>();
        try {
            merchantCoupon = marketingCouponService.getCoupon(uid, goodsIdCouponVos, allSkuVo, type);
        } catch (Exception e) {
            throw new RuntimeException("存在无效优惠券!!!");
        }
        Set<String> merchant = merchantCoupon.keySet();
        if (merchant.size() == 0) {
            merchant = merchantOrder.keySet();
        }
        for (String s : merchant) {
            Order order = merchantOrder.get(s);
            if (merchantCoupon.get(s) != null) {
                BigDecimal subtract = order.getAmount().subtract(merchantCoupon.get(s));
                order.setAmount(subtract.compareTo(BigDecimal.ZERO) == 1 ? subtract : BigDecimal.ZERO);
                order.setDiscountAmount(order.getDiscountAmount().add(merchantCoupon.get(s)));
                all = all.subtract(merchantCoupon.get(s));
                GoodsCoupon goodsCoupon = new GoodsCoupon();
                goodsCoupon.setCouponId("");
                goodsCoupon.setGoodsId("");
                goodsCoupon.setAmount(merchantCoupon.get(s));
                goodsCoupon.setOid(order.getOid());
                goodsCoupon.setUid(uid);
                goodsCouponService.save(goodsCoupon);
            }
            orderMapper.insert(order);
        }*/
        pay.setUid(uid);
        pay.setState(6);
        pay.setTotal(all);
        pay.setPaidAmount(BigDecimal.ZERO);
        pay.setUnpaidAmount(all);
        pay.setPayType(0);
        pay.setOidsJson(a.toJSONString());
        payService.save(pay);
        return pay.getPayId();
    }

   /* //创建订单
    @Transactional
    public String oldcreateOrder(String uid, OrderVo orderVo, Integer type) {
        //总价格
        BigDecimal all = BigDecimal.ZERO;
        //总共优惠
        BigDecimal couponAll = BigDecimal.ZERO;
        //总共促销
        BigDecimal promotionAll = BigDecimal.ZERO;
        //总共邮费
        BigDecimal postFeeAll = BigDecimal.ZERO;
        Pay pay = new Pay();
        pay.setPayId(OrderUtil.getPayId());
        JSONArray a = new JSONArray();
        List<MerchantGoodsVo> merchantGoodsVos = orderVo.getMerchantGoodsVos();
        for (MerchantGoodsVo m : merchantGoodsVos) {
            Order order = new Order();
            String oid = OrderUtil.getOid();
            order.setOid(oid);
            a.add(oid);
            //每家店的总价格
            BigDecimal merchantAll = BigDecimal.ZERO;
            //每个goods对应总价格
            Map<String, BigDecimal> goodsAll = new HashMap<>();
            //每个goods对应的商品数量
            Map<String, Integer> goodsSize = new HashMap<>();
            //每家店对应商品数量
            Integer merchantGoodSize = 0;
            //每家店对应商品重量
            BigDecimal merchantGoodWeight = BigDecimal.ZERO;
            //快递模板id
            String templateId = "";
            List<OrderSkuVo> orderSkuVos = m.getOrderSkuVos();
            for (OrderSkuVo orderSkuVo : orderSkuVos) {
                SkuVo skuVo = goodSkuService.getSkuVoBySkuId(orderSkuVo.getSkuId());
                if (skuVo == null || skuVo.getStatus() != 1) {
                    //商品不存在或下架
                    throw new RuntimeException("商品不存在或下架");
                }
                if (orderSkuVo.getNum() < 1) {
                    //商品数量错误
                    throw new RuntimeException("商品数量错误");
                }
                int i = goodSkuService.delStock(skuVo.getSkuId(), orderSkuVo.getNum());
                if (i != 1) {
                    throw new RuntimeException("库存不足!!!");
                }
                templateId = skuVo.getTemplateId();
                BigDecimal skuprice = skuVo.getOriginalPrice().multiply(new BigDecimal(orderSkuVo.getNum()));
                //统计总价格
                all = all.add(skuprice);
                //统计每家店价格
                merchantAll = merchantAll.add(skuprice);
                //统计每个goods对应总价格
                BigDecimal price = goodsAll.get(skuVo.getGoodsId());
                if (price == null) {
                    price = BigDecimal.ZERO;
                }
                price = price.add(skuprice);
                goodsAll.put(skuVo.getGoodsId(), price);
                //每个goods对应的总数量
                Integer gSize = goodsSize.get(skuVo.getGoodsId());
                if (gSize == null) {
                    gSize = 0;
                }
                gSize = gSize + orderSkuVo.getNum();
                goodsSize.put(skuVo.getGoodsId(), gSize);
                //统计每家店对应商品数量
                merchantGoodSize = merchantGoodSize + orderSkuVo.getNum();
                //统计每家店对应商品重量
                merchantGoodWeight = merchantGoodWeight.add(skuVo.getWeight().multiply(new BigDecimal(orderSkuVo.getNum())));
                //生成orderDetail
                OrderDetail orderDetail = new OrderDetail();
                orderDetail.setOid(order.getOid());
                orderDetail.setGoodsId(skuVo.getGoodsId());
                orderDetail.setSkuId(skuVo.getSkuId());
                orderDetail.setCoin(skuVo.getCoinName());
                orderDetail.setName(skuVo.getName());
                orderDetail.setNumber(orderSkuVo.getNum());
                orderDetail.setAmount(skuVo.getOriginalPrice());
                orderDetailService.save(orderDetail);
            }
            //每家店的总促销价格
            Map<String,BigDecimal>merchantPromotion = new HashMap<>();
            //merchantPromotion = marketingPromotionMapper.promotion(orderSkuVos);

            //每家店的总优惠价格
            BigDecimal mermantCoupon = BigDecimal.ZERO;
            List<GoodsIdCouponVo> goodsIdCouponVos = m.getGoodsIdCouponVos();
            for (GoodsIdCouponVo goodsIdCouponVo : goodsIdCouponVos) {
                boolean result = marketingCouponService.checkEffective(uid, goodsIdCouponVo.getCouponId(), type);
                if (!result) {
                    throw new RuntimeException("存在无效优惠券!!!");
                }
                String goodsId = marketingCouponService.getGoodsId(goodsIdCouponVo.getCouponId());
                BigDecimal price = marketingCouponService.getPrice(goodsIdCouponVo.getCouponId(), goodsAll.get(goodsId), goodsSize.get(goodsId), type);
                boolean remove = goodsIds.remove(goodsId);
                mermantCoupon = mermantCoupon.add(price);
                GoodsCoupon goodsCoupon = new GoodsCoupon();
                goodsCoupon.setCouponId(goodsIdCouponVo.getCouponId());
                goodsCoupon.setGoodsId(goodsIdCouponVo.getGoodsId());
                goodsCoupon.setAmount(price);
                goodsCoupon.setOid(order.getOid());
                goodsCoupon.setUid(uid);
                goodsCouponService.save(goodsCoupon);
            }
            //快递费
            BigDecimal PostFee = BigDecimal.ZERO;
            if (orderVo.getDelivery() == 0) {

            } else {
                PostFee = distributionTemplateService.getPrice(templateId, orderVo.getProvince(), merchantAll, merchantGoodSize, merchantGoodWeight);
            }

            if (PostFee.compareTo(new BigDecimal("-1")) == 0) {
                throw new RuntimeException("当前地区不支持快递");
            }
            postFeeAll = postFeeAll.add(PostFee);
            order.setUid(uid);
            order.setPayId(pay.getPayId());
            order.setMerchantId(m.getMerchantId());
            order.setTotal(merchantAll);
            order.setOrderType(1);
            order.setDelivery(orderVo.getDelivery());
            order.setUname(orderVo.getUname());
            order.setPhone(orderVo.getPhone());
            order.setAddress(orderVo.getAddress());
            order.setVNote(m.getVNote());
            order.setAmount(merchantAll.subtract(mermantPromotion).subtract(mermantCoupon).add(PostFee));
            //order.setDiscountAmount(promotionAll.add(couponAll));
            order.setPostFee(PostFee);
            order.setOrderState(7);
            orderMapper.insert(order);
        }
        pay.setUid(uid);
        pay.setState(6);
        pay.setTotal(all);
        pay.setPaidAmount(BigDecimal.ZERO);
        pay.setUnpaidAmount(all.subtract(couponAll).subtract(promotionAll).add(postFeeAll));
        pay.setPayType(0);
        pay.setOidsJson(a.toJSONString());
        payService.save(pay);
        return pay.getPayId();
    }*/

    @Override
    @Transactional
    public void handle(Object oid) {
        Order order = orderMapper.getByOid(oid.toString());
        System.out.println("head++++++" + oid.toString());
        //是否走链
        User user = userService.getByUid(order.getUid());
        boolean ischain = true;
        List<OrderDetail> details = orderDetailService.getList(order.getOid());
        if (!ischain) {
            //不是直接提货
            if (order.getDelivery() == 0) {
                for (OrderDetail orderDetail : details) {
                    //打币
                    UserAsset coin = userAssetService.getCoin(order.getUid(), orderDetail.getCoin());
                    if (coin == null) {
                        coin = new UserAsset();
                        coin.setCoin(orderDetail.getCoin());
                        coin.setAmount(new BigDecimal(orderDetail.getNumber()));
                        coin.setUid(order.getUid());
                        coin.setType(1);
                        userAssetService.save(coin);
                    } else {
                        userAssetService.addCoin(coin.getId(), new BigDecimal(orderDetail.getNumber()));
                    }
                    //记录
                    UserAssetRecord userAssetRecord = new UserAssetRecord();
                    userAssetRecord.setUid(order.getUid());
                    userAssetRecord.setAmount(new BigDecimal(orderDetail.getNumber()));
                    userAssetRecord.setCoin(orderDetail.getCoin());
                    userAssetRecord.setFlowId(OrderUtil.getFlowId());
                    userAssetRecord.setType(12);
                    userAssetRecord.setStatus(2);
                    userAssetRecord.setNote(order.getOid());
                    userAssetRecordService.save(userAssetRecord);
                    //加销量
                    countService.addSale(orderDetail.getGoodsId(), orderDetail.getNumber());
                    goodSkuService.addSales(orderDetail.getSkuId(), orderDetail.getNumber());
                }
            } else {
                Collection<Delivery> list = new ArrayList<>();
                for (OrderDetail orderDetail : details) {
                    //打币
                    UserAsset coin = userAssetService.getCoin(order.getUid(), orderDetail.getCoin());
                    if (coin == null) {
                        coin = new UserAsset();
                        coin.setCoin(orderDetail.getCoin());
                        coin.setFrozen(new BigDecimal(orderDetail.getNumber()));
                        coin.setUid(order.getUid());
                        coin.setType(1);
                        userAssetService.save(coin);
                    } else {
                        userAssetService.addUserfrozenAsset(coin.getId(), new BigDecimal(orderDetail.getNumber()));
                    }
                    //购买
                    UserAssetRecord userAssetRecord = new UserAssetRecord();
                    userAssetRecord.setUid(order.getUid());
                    userAssetRecord.setAmount(new BigDecimal(orderDetail.getNumber()));
                    userAssetRecord.setCoin(orderDetail.getCoin());
                    userAssetRecord.setFlowId(OrderUtil.getFlowId());
                    userAssetRecord.setType(12);
                    userAssetRecord.setNote(order.getOid());
                    userAssetRecord.setStatus(2);
                    userAssetRecordService.save(userAssetRecord);
                    //提货
                    UserAssetRecord userAssetRecord2 = new UserAssetRecord();
                    userAssetRecord2.setUid(order.getUid());
                    userAssetRecord2.setAmount(new BigDecimal(orderDetail.getNumber()));
                    userAssetRecord2.setCoin(orderDetail.getCoin());
                    userAssetRecord2.setFlowId(OrderUtil.getFlowId());
                    userAssetRecord2.setType(18);
                    userAssetRecord2.setStatus(2);
                    userAssetRecord2.setNote(order.getOid());
                    userAssetRecordService.save(userAssetRecord2);
                    Delivery delivery = new Delivery();
                    delivery.setOrderDeliveryId(OrderUtil.getDeliveryId());
                    delivery.setUid(order.getUid());
                    delivery.setMerchantId(order.getMerchantId());
                    delivery.setGoodsId(orderDetail.getGoodsId());
                    delivery.setSkuId(orderDetail.getSkuId());
                    delivery.setCoin(orderDetail.getCoin());
                    delivery.setName(orderDetail.getName());
                    delivery.setNumber(orderDetail.getNumber());
                    delivery.setUname(order.getUname());
                    delivery.setPhone(order.getPhone());
                    delivery.setAddress(order.getAddress());
                    delivery.setNote(orderDetail.getNote());
                    delivery.setDeliveryState(0);
                    list.add(delivery);
                    //加销量
                    countService.addSale(orderDetail.getGoodsId(), orderDetail.getNumber());
                    goodSkuService.addSales(orderDetail.getSkuId(), orderDetail.getNumber());
                }
                deliveryService.saveBatch(list);
            }
        } else {
            Merchant m = merchantService.info(order.getMerchantId());
            //不是直接提货
            if (order.getDelivery() == 0) {
                for (OrderDetail orderDetail : details) {
                    SkuVo skuVo = skuService.getBySkuId(orderDetail.getSkuId());
                    //打币
                    UserAsset coin = userAssetService.getCoin(order.getUid(), orderDetail.getCoin());
                    if (coin == null) {
                        coin = new UserAsset();
                        coin.setCoin(orderDetail.getCoin());
                        coin.setAmount(new BigDecimal(orderDetail.getNumber()));
                        coin.setUid(order.getUid());
                        coin.setType(1);
                        userAssetService.save(coin);
                    } else {
                        userAssetService.addCoin(coin.getId(), new BigDecimal(orderDetail.getNumber()));
                    }
                    log.error("skuVo+++"+skuVo.toString());
                    //String hash = paraChainUtil.evmTransfer(, null, user.getAddress(), , orderDetail.getCoin());
                    //String replace = orderDetail.getCoin().replace(MALLGlobalConfig.GOODS_NFT_PREFIX,"");
                    String hash = tranCoinAndNft(skuVo.getSkuId(),m.getTokenIssueAddr(),user.getAddress(),new BigDecimal(orderDetail.getNumber()),skuVo.getType(),orderDetail.getCoin());
                    //记录
                    UserAssetRecord userAssetRecord = new UserAssetRecord();
                    userAssetRecord.setUid(order.getUid());
                    userAssetRecord.setAmount(new BigDecimal(orderDetail.getNumber()));
                    userAssetRecord.setCoin(orderDetail.getCoin());
                    userAssetRecord.setFlowId(OrderUtil.getFlowId());
                    userAssetRecord.setType(12);
                    userAssetRecord.setStatus(2);
                    userAssetRecord.setNote(order.getOid());
                    userAssetRecordService.save(userAssetRecord);
                    //加销量
                    countService.addSale(orderDetail.getGoodsId(), orderDetail.getNumber());
                    goodSkuService.addSales(orderDetail.getSkuId(), orderDetail.getNumber());
                    //转币
                    TranDto tranDto = new TranDto();
                    tranDto.setId(userAssetRecord.getId());
                    tranDto.setAmount(new BigDecimal(orderDetail.getNumber()));
                    //tranDto.setCoin(aLong.toString());
                    tranDto.setFromAddr(m.getNftAddr());
                    tranDto.setHash(hash);
                    tranDto.setToAddr(user.getAddress());
                    tranDto.setDelivery(order.getDelivery() == 1);
                    tranDto.setRecoAddr("");
                    tranDto.setPrice(orderDetail.getAmount());
                    tranDto.setUpdateTime(System.currentTimeMillis());
                    log.error(JSONObject.toJSONString(tranDto));
                    msgProducer.sendTTL(JSONObject.toJSONString(tranDto), QueueTTLTypeEnum.CONFIRM_WITHDRAW_TTL, 5 * 1000);
                }
            } else {
                Collection<Delivery> list = new ArrayList<>();
                for (OrderDetail orderDetail : details) {
                    //打币
                    SkuVo skuVo = skuService.getBySkuId(orderDetail.getSkuId());
                    String hash = tranCoinAndNft(skuVo.getSkuId(),m.getTokenIssueAddr(),user.getAddress(),new BigDecimal(orderDetail.getNumber()),skuVo.getType(),orderDetail.getCoin());
                    UserAsset coin = userAssetService.getCoin(order.getUid(), orderDetail.getCoin());
                    if (coin == null) {
                        coin = new UserAsset();
                        coin.setCoin(orderDetail.getCoin());
                        coin.setFrozen(new BigDecimal(orderDetail.getNumber()));
                        coin.setUid(order.getUid());
                        coin.setType(1);
                        userAssetService.save(coin);
                    } else {
                        userAssetService.addUserfrozenAsset(coin.getId(), new BigDecimal(orderDetail.getNumber()));
                    }
                    //记录
                    UserAssetRecord userAssetRecord = new UserAssetRecord();
                    userAssetRecord.setUid(order.getUid());
                    userAssetRecord.setAmount(new BigDecimal(orderDetail.getNumber()));
                    userAssetRecord.setCoin(orderDetail.getCoin());
                    userAssetRecord.setFlowId(OrderUtil.getFlowId());
                    userAssetRecord.setType(12);
                    userAssetRecord.setStatus(2);
                    userAssetRecord.setNote(order.getOid());
                    userAssetRecordService.save(userAssetRecord);

                    //提货
                    UserAssetRecord userAssetRecord2 = new UserAssetRecord();
                    userAssetRecord2.setUid(order.getUid());
                    userAssetRecord2.setAmount(new BigDecimal(orderDetail.getNumber()));
                    userAssetRecord2.setCoin(orderDetail.getCoin());
                    userAssetRecord2.setFlowId(OrderUtil.getFlowId());
                    userAssetRecord2.setType(18);
                    userAssetRecord2.setStatus(2);
                    userAssetRecord2.setNote(order.getOid());
                    userAssetRecordService.save(userAssetRecord2);
                    Delivery delivery = new Delivery();
                    delivery.setOrderDeliveryId(OrderUtil.getDeliveryId());
                    delivery.setUid(order.getUid());
                    delivery.setMerchantId(order.getMerchantId());
                    delivery.setGoodsId(orderDetail.getGoodsId());
                    delivery.setSkuId(orderDetail.getSkuId());
                    delivery.setCoin(orderDetail.getCoin());
                    delivery.setName(orderDetail.getName());
                    delivery.setNumber(orderDetail.getNumber());
                    delivery.setUname(order.getUname());
                    delivery.setPhone(order.getPhone());
                    delivery.setAddress(order.getAddress());
                    delivery.setNote(orderDetail.getNote());
                    delivery.setDeliveryState(0);
                    list.add(delivery);
                    //转币
                    TranDto tranDto = new TranDto();
                    tranDto.setId(userAssetRecord.getId());
                    tranDto.setAmount(new BigDecimal(orderDetail.getNumber()));
                    //tranDto.setCoin(aLong.toString());
                    tranDto.setFromAddr(m.getNftAddr());
                    tranDto.setHash(hash);
                    tranDto.setToAddr(user.getAddress());
                    tranDto.setDelivery(order.getDelivery() == 1);
                    tranDto.setRecoAddr(m.getTokenRecoverAddr());
                    tranDto.setPrice(orderDetail.getAmount());
                    tranDto.setUpdateTime(System.currentTimeMillis());
                    log.error(JSONObject.toJSONString(tranDto));
                    msgProducer.sendTTL(JSONObject.toJSONString(tranDto), QueueTTLTypeEnum.CONFIRM_WITHDRAW_TTL, 5 * 1000);
                    //加销量
                    countService.addSale(orderDetail.getGoodsId(), orderDetail.getNumber());
                    goodSkuService.addSales(orderDetail.getSkuId(), orderDetail.getNumber());
                }
                deliveryService.saveBatch(list);
            }
        }


    }

    @Override
    @Transactional
    public void close(Object oid) {
        Order order = orderMapper.getByOid(oid.toString());
        if (order == null) {
            log.error(oid.toString());
            return;
        }
        order.setOrderState(2);
        orderMapper.updateById(order);
        List<OrderDetail> list = orderDetailService.getList(oid.toString());
        for (OrderDetail orderDetail : list) {
            //加库存
            goodSkuService.addStock(orderDetail.getSkuId(), orderDetail.getNumber());
        }
    }

    @Override
    @Transactional
    public int applyReturn(String uid, String oid) {
        List<OrderDetail> list = orderDetailService.getList(oid);
        for (OrderDetail orderDetail : list) {
            //冻结
            int i = userAssetService.frozenUserAssets(uid, orderDetail.getCoin(), orderDetail.getNumber());
            if (i != 1) {
                throw new RuntimeException("asset_not_enough");
            }
        }
        return orderMapper.applyReturn(uid, oid, System.currentTimeMillis());
    }

    @Override
    @Transactional
    public int revokeReturn(String uid, String oid) {
        refundMapper.revokeReturn(oid, System.currentTimeMillis());
        //解冻资产
        List<OrderDetail> list = orderDetailService.getList(oid);
        for (OrderDetail orderDetail : list) {
            int i = userAssetService.unfrozenUserAsset(uid, orderDetail.getNumber(), orderDetail.getCoin());
            if (i != 1) {
                throw new RuntimeException("frozen_asset_not_enough");
            }
        }

        return orderMapper.revokeReturn(uid, oid, System.currentTimeMillis());
    }

    @Override
    public ReturnMoneyDetail getReturnMoneyDetail(String oid) {
        ReturnMoneyDetail returnMoneyDetail = orderMapper.getReturnMoneyDetail(oid);
        if (returnMoneyDetail == null) {
            return null;
        }
        List<SkuVo> skuVos = new ArrayList<>();
        List<OrderDetail> list = orderDetailService.getList(oid);
        for (OrderDetail orderDetail : list) {
            SkuVo skuVoBySkuId = skuService.getBySkuId(orderDetail.getSkuId());
            skuVos.add(skuVoBySkuId);
        }
        returnMoneyDetail.setList(skuVos);
        return returnMoneyDetail;
    }

    @Override
    public Integer getCountByUidAndOid(String uid, String orderId) {
        QueryWrapper<Order> q = new QueryWrapper<>();
        q.eq("uid", uid);
        q.eq("oid", orderId);
        return count(q);
    }

    @Override
    @Transactional
    public String createPreSaleOrder(String uid, PreSaleOrderVo preSaleOrderVo) {
        SkuVo skuVo = goodSkuService.getSkuVoBySkuId(preSaleOrderVo.getSkuId());
        PreSale preSale = preSaleService.getByGoodsId(skuVo.getGoodsId());
        int i = goodSkuService.delStock(preSaleOrderVo.getSkuId(), preSaleOrderVo.getNumber());
        if (i != 1) {
            throw new RuntimeException("库存不足!!!");
        }
        JSONArray a = new JSONArray();
        Pay pay = new Pay();
        pay.setPayId(OrderUtil.getPayId());
        Order order = new Order();
        order.setOid(OrderUtil.getOid());
        a.add(order.getOid());
        order.setUid(uid);
        order.setPayId(pay.getPayId());
        order.setMerchantId(skuVo.getMerchantId());
        order.setTotal(skuVo.getOriginalPrice().multiply(new BigDecimal(preSaleOrderVo.getNumber())));
        order.setOrderType(2);
        order.setPostFee(BigDecimal.ZERO);
        order.setDiscountAmount(skuVo.getOriginalPrice());
        if (preSale.getDepositType() == 1) {
            order.setAmount(skuVo.getOriginalPrice().multiply(preSale.getDeposit().multiply(new BigDecimal(preSaleOrderVo.getNumber()))).setScale(2, RoundingMode.HALF_UP));
        } else {
            order.setAmount(preSale.getDeposit().multiply(new BigDecimal(preSaleOrderVo.getNumber())).setScale(2, RoundingMode.HALF_UP));
        }

        order.setOrderState(4);
        save(order);
        //订单详情
        OrderDetail orderDetail = new OrderDetail();
        orderDetail.setOid(order.getOid());
        orderDetail.setAmount(skuVo.getOriginalPrice());
        orderDetail.setGoodsId(skuVo.getGoodsId());
        orderDetail.setSkuId(skuVo.getSkuId());
        orderDetail.setName(skuVo.getName());
        orderDetail.setCoin(skuVo.getCoinName());
        orderDetail.setNumber(preSaleOrderVo.getNumber());
        orderDetailService.save(orderDetail);
        pay.setUid(uid);
        pay.setState(4);
        pay.setTotal(order.getAmount());
        pay.setUnpaidAmount(order.getAmount());
        pay.setPaidAmount(BigDecimal.ZERO);
        pay.setPayType(4);
        pay.setOidsJson(a.toJSONString());
        payService.save(pay);
        return pay.getPayId();
    }

    @Override
    @Transactional
    public String createNftOrder(String uid, List<NftOrderPo> nftOrderPos) {
        BigDecimal amount = BigDecimal.ZERO;
        //merchant-goodIds
        Map<String, List<String>> shopGoodsList = new HashMap<>();
        //goodIds-number
        Map<String, Integer> goodsIdNumber = new HashMap<>();
        //goodsId-amount
        Map<String, BigDecimal> goodIdsAmount = new HashMap<>();
        //支付单
        Pay pay = new Pay();
        pay.setPayId(OrderUtil.getPayId());
        for (NftOrderPo nftOrderPo : nftOrderPos) {
            //商家归类
            GoodsVo byGoodId = ispuService.getByGoodId(nftOrderPo.getGoodsId());
            List<String> list = shopGoodsList.get(byGoodId.getMerchantId());
            if (list == null) {
                list = new ArrayList<>();
                list.add(nftOrderPo.getGoodsId());
            } else {
                list.add(nftOrderPo.getGoodsId());

            }
            shopGoodsList.put(byGoodId.getMerchantId(), list);
            goodsIdNumber.put(nftOrderPo.getGoodsId(), nftOrderPo.getNumber());
        }
        Set<String> strings = shopGoodsList.keySet();
        for (String merchantId : strings) {
            Order order = new Order();
            order.setOid(OrderUtil.getOid());
            order.setUid(uid);
            order.setPayId(pay.getPayId());
            order.setMerchantId(merchantId);
            //order.setTotal(skuVo.getOriginalPrice().multiply(new BigDecimal(preSaleOrderVo.getNumber())));
            BigDecimal orderAmount = BigDecimal.ZERO;
            order.setOrderType(2);
            List<String> list = shopGoodsList.get(merchantId);
            for (String goodsId : list) {
                orderAmount = orderAmount.add(goodIdsAmount.get(goodsId));
                OrderDetail orderDetail = new OrderDetail();
                orderDetail.setNumber(goodsIdNumber.get(goodsId));
                Nft byGoodsId = nftService.getByGoodsId(goodsId);
                orderDetail.setCoin(byGoodsId.getTokenId());
            }

        }
        //减库存
       /* Nft byGoodsId = nftService.getByGoodsId(nftOrderPo.getGoodsId());
        if(byGoodsId == null){
            throw new RuntimeException("GOODS_NOT_EXIST");
        }
        int i = nftService.delStock(byGoodsId.getId(),nftOrderPo.getNumber(),byGoodsId.getUpdateTime(),System.currentTimeMillis());
        if(i != 1){
            throw new RuntimeException("STOCK_NOT_ENOUGH");
        }*/
        return null;
    }

    public static void main(String[] args) {
        String str = "NTF-128938890";
        str = str.replace("NTF-", "");
        System.out.println(str);
    }

    private String tranCoinAndNft(String skuId,String from,String to,BigDecimal number,int goodsType,String coin) {
        //商品通证(0.否 1.是，自动生成Token 2.是，使用外部既有Token)
        int commodity = spuService.getCommodityBySkuId(skuId);
        if(goodsType == 1||goodsType == 2){
            if(commodity == 1){
                return chainUtil.tranToken(from,"",to,number,coin);
            }else if(commodity == 2){
                Long trueCoin = skuService.getTrueCoin(coin);
                long [] a = new long[]{trueCoin};
                long [] b = new long[]{number.longValue()};
                return  chainUtil.evmExec(a,b,from,to);
            }
        }else if(goodsType == 6){
            if(commodity == 1){
                String coinName = coin.replace(MALLGlobalConfig.GOODS_NFT_PREFIX, "");
                return chainUtil.evmTransfer(to,from,Long.valueOf(coinName),number.longValue());
            }else if(commodity == 2){
                Long trueCoin = skuService.getTrueCoin(coin);
                long [] a = new long[]{trueCoin};
                long [] b = new long[]{number.longValue()};
                return  chainUtil.evmExec(a,b,from,to);
            }
        }
        return "";
    }
}
